/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.francl.eikon;

import org.francl.eikon.database.Database;
import org.francl.eikon.database.DatabaseFactory;
import org.francl.eikon.util.Configuration;
import org.francl.eikon.util.ImageMetaData;
import org.francl.eikon.image.ImageDownloader;
import org.francl.eikon.wavelet.Signature;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

/** Loads a bunch of images from a file full of URLs. The file format is:
    <blockquote>
    url1<br>
    url2<br>
    ...<br>
    urln<br>
    </blockquote>
    
    @author Luke Francl
**/

public class BulkLoader {
    
    public static void main(String args[]) {
	
	if(args.length != 1) {
	    System.out.println("Usage: java BulkLoader urlFile");
	    System.exit(0);
	}
	
	try {
	    Configuration config = Configuration.getInstance();
	    String save = config.get("database.save.file");
	    if (save == null || save.equals("")) {
		String m = "Blah, no config for database.save.file";
		throw new IllegalArgumentException(m);
	    }

	    File saveFile = new File(save);
		
	    Database db = DatabaseFactory.getDatabase();
	    
	    db.retrieveDatabase(saveFile);

	    String surl = new String();
	    BufferedReader urlinput = new BufferedReader(new 
		FileReader(args[0]));
	    
	    while((surl = urlinput.readLine()) != null) {
		
		System.out.println("Trying to add " + surl);
		try {
		    ImageDownloader downloadNewImage =
			new ImageDownloader(surl);
		    Signature sig = downloadNewImage.getSignature();
		    ImageMetaData imd = downloadNewImage.getImageMetaData();
		    imd.setAverage('Y',sig.getAverage('Y'));
		    imd.setAverage('I',sig.getAverage('I'));
		    imd.setAverage('Q',sig.getAverage('Q'));
		    
		    // add the image if it isn't already in the 'database'
		    if(!db.contains(imd)) {
			db.addImage(imd, sig);
		    } else {
			System.out.println("Database already contains " + imd);
		    }
		} catch (Exception e) {
		    e.printStackTrace();
		    //but keep on going!
		}
	    }

	    // Save the ImageMetaDataCollection out to disk
	    db.saveDatabase(saveFile);
	} catch (Exception e) {
	    e.printStackTrace(System.err);
	}
    }
}

