/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon;

import org.francl.eikon.database.Database;
import org.francl.eikon.database.DatabaseFactory;
import org.francl.eikon.image.ImageDownloader;
import org.francl.eikon.util.ImageMetaData;
import org.francl.eikon.util.Configuration;
import org.francl.eikon.query.Query;
import org.francl.eikon.query.ResultSet;
import org.francl.eikon.wavelet.Signature;

import java.io.File;

public class Main {

    public static void main(String[] args) {
	
	if(args.length != 2) {
	    System.out.println("Usage: java Main url number_of_results");
	    System.exit(0);
	}

	String surl = args[0];
	int numResults;
	Database db;
	try {
	    numResults = Integer.parseInt(args[1]);
	    db = DatabaseFactory.getDatabase();

	    Configuration config = Configuration.getInstance();
	    String save = config.get("database.save.file");

	    File saveFile = new File(save);
	    
	    db.retrieveDatabase(saveFile);

	    ImageDownloader downLoader = new ImageDownloader(surl);
	    Signature sig = downLoader.getSignature();
	    ImageMetaData imd = downLoader.getImageMetaData();
	    imd.setAverage('Y',sig.getAverage('Y'));
	    imd.setAverage('I',sig.getAverage('I'));
	    imd.setAverage('Q',sig.getAverage('Q'));

	    Query query = new Query(sig, numResults);
	    ResultSet rs = query.getResultSet(imd.getResourceId());
	    System.out.println(rs.toString());

	    // TODO: this needs to save the DB back to disk.
	    
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}    
