/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.database;

import org.francl.eikon.wavelet.Signature;
import org.francl.eikon.wavelet.ColorChannelNotSupportedException;
import org.francl.eikon.util.ResourceId;
import org.francl.eikon.util.ImageMetaData;

import java.util.Collection;
import java.util.Iterator;

import java.io.File;
import java.io.IOException;

/** The Database abstract class provides methods for inserting, removing, and
    querying the database that stores image metadata and signatures.
    <p>Classes that implement Database are all Singletons, and should not
    define public constructors. However, since this is an interface, I cannot
    define a static getInstance method. Implementors should take the
    responsibility to define a getInstance method and make the constructor
    private.

    <p>And the developer (i.e., Me should take the effort to define how this
    should work in a better way :)

    <p>TODO: Determine if this fix is appopriate:
    http://members.tripod.com/rwald/java/articles/Singleton_in_Java.html
**/
public interface Database {
    /** Do I need this operation? It'll increase storage. do i use it?**/
    public boolean contains(ImageMetaData imd);
    public Signature getSignature(ResourceId hash);
    public ImageMetaData getImageMetaData(ResourceId hash);
    public Iterator getAllImageMetaData();
    public void addImage(ImageMetaData imd, Signature sig)
	throws ColorChannelNotSupportedException;
    public void removeImage(ResourceId hash);
    public void updateMetaData(ImageMetaData imd)
	throws ImageMetaDataNotFoundException;
    public Collection matches(int xPos, int yPos, String color,
				       int value);
    /** Low priority to get this one done... **/
    public Collection matches(int xPos, int yPos, String color,
				       int value, String basis, int width,
				       int height);
    /** Optional operation, used for file-based databases.
	@throws IOException if an error occurs writing the
	serialized database to disk.
	@throws UnsupportedOperationException if this database doesn't
	support this operation.
     **/
    public void saveDatabase(File file) throws IOException;
    /** Optional operation, used for file-based databases.
	@throws IOException if an error occurs reading the
	seralized database from the disk.
	@throws UnsupportedOperationException if this database doesn't
	support this operation.
    **/
    public void retrieveDatabase(File file) throws IOException;

}

