/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.database;

import org.francl.eikon.util.SearchArray;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

/** There are 6 SearchArrays for each colorspace, and this class holds them
    and gives the programmer easy access to them. There are two SearchArrays
    for each color, one for the positive coeffients and the other for the
    negative coeffients.
**/
public class SearchArrayCollection implements Serializable {

    public static final String POSITIVE = "positive";
    public static final String NEGATIVE = "negative";

    /** maps colors to another hashmap, which maps "negative" and
	"positive" to and actual SearchArray.
	<p>For example: Color -> Pos -> aSearchArray
    **/
    private HashMap searchArrays;

    public SearchArrayCollection() {
	System.out.println("SearchArrayCollection: creating new.");
	searchArrays = new HashMap();
    }

    /** gets all the colors that are stored in the SearchArrayCollection **/
    public Iterator getColors() {
	return searchArrays.keySet().iterator();
    }

    /** Gets the SearchArray for the specified color and sign. 
	@param color a one character long string representing the color channel
	for the SearchArray
	@param sign the sign of the SearchArray. Please use
	SearchArray.POSITIVE or SearchArray.NEGATIVE.
	@returns the SearchArray for the tuple (color, sign). 
	@throws IllegalArgumentException if color is more than one character
	long
    **/
    public SearchArray getSearchArray(String color, String sign) {
	if(color.length() > 1) {
	    String m = "Color (" + color + ") was > 1 character. " +
		"This is not what you want.";
	    throw new IllegalArgumentException(m);
	}

	if (!sign.equals(NEGATIVE) && !sign.equals(POSITIVE)) {
	    String m = "Sign was " + sign + " but it must be one of " +
		POSITIVE + " or " + NEGATIVE;
	    throw new IllegalArgumentException(m);
	}

	if(!searchArrays.containsKey(color)) {
	    String m = "SearchArrayCollection.getSearchArray: Creating new " +
		"color -> sign mapping for " + color;
	    System.out.println(m);
	    searchArrays.put(color, new HashMap());
	}

	HashMap colorMap = (HashMap)searchArrays.get(color);
	if(!colorMap.containsKey(sign)) {
	    String m = "SearchArrayCollection.getSearchArray: Creating new " +
		"sign -> SearchArray mapping for " + sign;
	    System.out.println(m);
	    colorMap.put(sign, new SearchArray());
	}
	
	return (SearchArray)colorMap.get(sign);
    }
}
