/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.database;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.Collection;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

import org.francl.eikon.wavelet.Signature;
import org.francl.eikon.wavelet.ColorChannelNotSupportedException;
import org.francl.eikon.util.ImageMetaData;
import org.francl.eikon.util.ResourceId;


public class SerializedDatabase implements Database {
    private static SerializedDatabase singleton   = null;
    /** Don't use me until you call retrieveDatabase! **/
    private ImageMetaDataCache imageMdCache       = null; 

    private SerializedDatabase() { }

    public static Database getInstance() {
	if (singleton == null) {
	    singleton = new SerializedDatabase();
	}

	return singleton;
    }

    public boolean contains(ImageMetaData imd) {
	return imageMdCache.contains(imd);
    }

    public Signature getSignature(ResourceId hash) {
	throw new UnsupportedOperationException("Not done yet");
    }

    public ImageMetaData getImageMetaData(ResourceId id) {
	return imageMdCache.getImageMetaData(id);
    }

    public Iterator getAllImageMetaData() {
	return imageMdCache.getAllImageMetaData();
    }

    public void addImage(ImageMetaData imd, Signature sig)
	throws ColorChannelNotSupportedException {
	imageMdCache.addImage(imd, sig);
    }

    public void removeImage(ResourceId id) {
	imageMdCache.removeImage(id);
    }

    public void updateMetaData(ImageMetaData imd)
	throws ImageMetaDataNotFoundException {
	imageMdCache.updateMetaData(imd);
    }

    public Collection matches(int xPos, int yPos, String color, int value) {
	return imageMdCache.matches(xPos, yPos, color, value);
    }

    public Collection matches(int xPos, int yPos, String color, int value,
			      String basis, int width, int height) {
	throw new UnsupportedOperationException("Not done yet.");
    }
    
    public void saveDatabase(File file) throws IOException {
	ObjectOutputStream out =
	    new ObjectOutputStream(new FileOutputStream(file));
	out.writeObject(imageMdCache);
    }

    public void retrieveDatabase(File file) throws IOException {
	System.out.println("Trying to load db file: " + file);
	if(!file.exists()) {
	    imageMdCache = new ImageMetaDataCache();
	    return;
	}

	try {
	    ObjectInputStream in =
		new ObjectInputStream(new FileInputStream(file));
	    imageMdCache = (ImageMetaDataCache) in.readObject();
	} catch (ClassNotFoundException e) {
	    e.printStackTrace();
	    System.out.println("Error loading database, creating a new one.");
	    imageMdCache = new ImageMetaDataCache();
	}

	System.out.println("Loaded cache: " + imageMdCache.toString());
    }
}
