/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.query;

import org.francl.eikon.util.ResourceId;
import org.francl.eikon.database.Database;
import org.francl.eikon.database.DatabaseFactory;
import org.francl.eikon.util.ImageMetaData;

import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

public class ResultSet {

    /** This needs to be ordered! **/
    private ArrayList results = new ArrayList();
    private Database db;
    private ImageMetaData exact;
    private Collection resourceIds;

    /** Use this ResultSet when you don't know the image being used as
	a query -- there will be no exact match in this instance. **/
    public ResultSet(Collection Ids) {
	resourceIds = Ids;
	db = DatabaseFactory.getDatabase();
	exact = null;
	fillResultSet(resourceIds);
    }
    
    /** Use this ResultSet when you know the image being used as a query
	for an exact match **/
    public ResultSet(Collection Ids, ResourceId queryImage) {
	resourceIds = Ids;
	db = DatabaseFactory.getDatabase();
	exact = db.getImageMetaData(queryImage);
	fillResultSet(resourceIds);
    }

    private void fillResultSet(Collection Ids) {
	Iterator it = Ids.iterator();
	while(it.hasNext()) {
	    ResourceId currentId = (ResourceId)it.next();
	    ImageMetaData imd = db.getImageMetaData(currentId);
	    results.add(imd);
	}
    }

    public Collection getResults() {
	return results;
    }
    
    public Iterator iterator() {
	return results.iterator();
    }

    /** Should use a StringBuffer for this for better concatenation
	preformance, eh?
    **/
    public String toString() {
	String s = "";
	if (exact != null) {
	    s += "Exact Match:\n";
	    s += "------------\n";
	    s += exact.toString();
	}

	s += "Top " + resourceIds.size() + " Matches\n";
	s += "----" + "-" +                "--------\n";
	Iterator i = results.iterator();
	while(i.hasNext()) {
	    ImageMetaData current = (ImageMetaData)i.next();
	    s += current.toString();
	}

	return s;
    }

    /** Should use a StringBuffer for this for better concatenation
	preformance, eh?
    **/
    public String toXml() {
	String s;
	s  = "<?xml-stylesheet type=\"text/xsl\" href=\"results.xsl\"?>\n";
	s += "<eikonmatches>\n";

	if (exact != null) {
	    s += "<exactmatch>\n";
	    s += exact.toXml();
	    s += "</exactmatch>\n";
	}

	Iterator i = results.iterator();
	while (i.hasNext()) {
	    ImageMetaData current = (ImageMetaData)i.next();
	    s += current.toXml();
	}

	s += "</eikonmatches>\n";

	return s;
		
    }
}
