/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.francl.eikon.servlet;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.File;

import org.francl.eikon.database.Database;
import org.francl.eikon.database.DatabaseFactory;
import org.francl.eikon.util.Configuration;

/** The EikonServlet handles instantiating the DB and saving it on
    shut down, so the other servlets don't have to worry about it.
    No copy-and-paste coding here!
    @author Luke Francl
**/
public class EikonServlet extends HttpServlet {
    
    /** This is safe to have as a class variable because everything is
	synchronized **/
    protected Database db = null;
    
    public void init() throws ServletException {
	try {
	    Configuration config = Configuration.getInstance();
	    File saveFile = new File(config.get("database.save.file"));
	    db = DatabaseFactory.getDatabase();
	    db.retrieveDatabase(saveFile);
	} catch (Exception e) {
	    throw new ServletException(e);
	}
    }

    /** Serialize the database so we can save all of that yummy goodness **/
    public void destroy() {
	try {
	    Configuration config = Configuration.getInstance();
	    File saveFile = new File(config.get("database.save.file"));
	    db.saveDatabase(saveFile);
	} catch (Exception e) {
	    // At this point, there's really nothing I can do
	    e.printStackTrace();
	}
    }
}
