/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.servlet;

import org.francl.eikon.database.Database;
import org.francl.eikon.image.ImageDownloader;
import org.francl.eikon.wavelet.Signature;
import org.francl.eikon.util.ImageMetaData;

import java.io.PrintWriter;
import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;


public class LoaderServlet extends EikonServlet {

    public void doGet(HttpServletRequest req, HttpServletResponse res)
	throws IOException {
	String url      = req.getParameter("url");
	res.setContentType("text/text");
	PrintWriter out = res.getWriter();

	try {
	    ImageDownloader dl = new ImageDownloader(url);
	    Signature sig      = dl.getSignature();
	    ImageMetaData imd  = dl.getImageMetaData();
	    imd.setAverage('Y', sig.getAverage('Y'));
	    imd.setAverage('I', sig.getAverage('I'));
	    imd.setAverage('Q', sig.getAverage('Q'));

	    if(db.contains(imd)) {
		out.print("Database already contains image at " + url);
	    } else {
		db.addImage(imd, sig);
		out.print("Image at " + url + " added successfully");
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
			  e.toString());
	}
    }
}
