/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.servlet;

import org.francl.eikon.database.Database;
import org.francl.eikon.image.ImageDownloader;
import org.francl.eikon.wavelet.Signature;
import org.francl.eikon.util.ImageMetaData;
import org.francl.eikon.query.Query;
import org.francl.eikon.query.ResultSet;

import java.io.PrintWriter;
import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;

public class UrlQueryServlet extends EikonServlet {

    public static final int DEFAULT_NUM_RESULTS = 20;
    
    public void doGet(HttpServletRequest req, HttpServletResponse res)
	throws IOException {
	String queryUrl   = req.getParameter("queryUrl");
	String numResults = req.getParameter("numResults");
	res.setContentType("text/xml");
	PrintWriter out   = res.getWriter();
	int results = DEFAULT_NUM_RESULTS;
	try {
	    results = Integer.parseInt(numResults);
	    if (results < 0 || results > 20) {
		results = DEFAULT_NUM_RESULTS;
	    }
	} catch (Exception e) {}
	    

	try {

	    ImageDownloader dl = new ImageDownloader(queryUrl);
	    Signature sig = dl.getSignature();
	    ImageMetaData imd = dl.getImageMetaData();
	    imd.setAverage('Y',sig.getAverage('Y'));
	    imd.setAverage('I',sig.getAverage('I'));
	    imd.setAverage('Q',sig.getAverage('Q'));

	    Query query = new Query(sig, results);
	    ResultSet rs = query.getResultSet(imd.getResourceId());
	    out.print(rs.toXml());
	    // add the image if it isn't already in the database
	    if(!db.contains(imd)) {
		db.addImage(imd, sig);
	    } else {
		System.out.println("Database already contains " +imd);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    //TODO: make this print the whole stack trace..
	    res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
			  e.toString());
	}
    }
}
