package org.francl.eikon.util;

import org.francl.eikon.wavelet.ColorChannelNotSupportedException;

import java.io.Serializable;
import java.net.URL;
import java.net.URLEncoder;

/** ImageMetaData objects hold ... image meta data. :)
    In this implementation, this consists of the following properties:
    <ul>
    <li>width
    <li>height
    <li>source URL -- didn't I want this to be a List now?
    <li>image MIME type 
    <li>ResourceId -- SHA message digest (hash) -- 160 bit unique identifier
    </ul>
    @author Luke Francl
    TODO: This class isn't colorchannel agnostic!
**/
public class ImageMetaData implements Serializable {
    protected int width;
    protected int height;
    protected URL url;
    protected String imageType;
    protected ResourceId id;

    protected double Yavg;
    protected double Iavg;
    protected double Qavg;

    public  void setWidth(int x) {
	this.width = x;
    }

    public  void setHeight(int y) {
	this.height = y;
    }
    
    public   void setURL(URL u) {
	this.url = u;
    }
	
    public   void setImageType(String type) {
	this.imageType = type;
    }

    public void setResourceId(ResourceId id) {
	this.id = id;
    }

    public int getWidth() {
	return this.width;
    }

    public int getHeight() {
	return this.height;
    }

    public URL getUrl() {
	return this.url;
    }
    
    public String getImageType() {
	return this.imageType;
    }

    public ResourceId getResourceId() {
	return this.id;
    }

    public double getAverage(String color) {
	return getAverage(Util.stringToChar(color));
    }

    public double getAverage(char color) {

	double retval;
	if(color == 'Y') {
	    return this.Yavg;
	} else if(color == 'I') {
	    return this.Iavg;
	} else {
	    return this.Qavg;
	}
    }

    public void setAverage(String color, double avg) {
	setAverage(Util.stringToChar(color), avg);
    }

    /** It is VITALLY important that the average color get set
	for an ImageMetaData object stored in the database to be remotely
	useful...**/
    public void setAverage(char color, double avg) {
	switch(color) {
	case 'Y':
	    Yavg = avg;
	    break;
	case 'I':
	    Iavg = avg;
	    break;
	case 'Q':
	    Qavg = avg;
	    break;
	default:
	    throw new ColorChannelNotSupportedException("Oops: " + color);
	}
    }

    /** To add ImageMetaData objects to a HashMap I Must override 
	hashCode ... 
    **/
    public int hashCode() {
	return this.id.hashCode();
    }

    /** ...and equals **/
    public  boolean equals(Object o) {
	if ((o == null) || ! (o instanceof ResourceId)) {
	    return false;
	} 
	
	if (this.id.equals((ResourceId)o)) {
	    return true;
	} else {
	    return false;
	}
    }
	    
    public String toString() {
	String s;
	s  = "Image Metadata for " + id.toString() + "\n";;
	s += "Width = " + width + " ";
	s += "Height = " + height + "\n"; ;
	s += "URL: " + url + "\n";
	s += "Type: " + imageType + "\n";

	return s;
    }


    /** This is for my hack-y Perl (also a hack-y name; I'm actually using
	PHP not perl. Hey, it was 4 in the morning.) pass-through client 
	that prints HTML to be included on a webpage.**/
    public String toHTMLString() {
	String s;
	s = "<p><b>SHA Hash:</b> " + id.toString() + "<br>";
	s += "<b>Width:</b> " + width + "<br>";
	s += "<b>Height:</b> " + height + "<br>";
	s += "<b>URL:</b> <a href=" + url + ">" + url + "</a><br>";
	s += "<b><a href=query?QueryURL=" + URLEncoder.encode(url.toString()) +">Use this image as a query</a></b><br>";
	s += "<img src="+ url +">";
	s += "<hr>";

	return s;
    }

    public String toXml() {
	String s;
	s  = "<imagemetadata>\n";
	s += "<resourceid>" + id.toString() + "</resourceid>\n";
	s += "<source>" + Util.escapeAmp(url.toString()) + "</source>\n";
	s += "<source-encoded>" + URLEncoder.encode(url.toString()) +
	    "</source-encoded>\n";
	s += "<width>" + width + "</width>\n";
	s += "<height>" + height + "</height>\n";
	s += "</imagemetadata>\n";
	    
	return s;
    }
	    

}
