/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.util;

// This is all under the GPL (c) OpenCola, Inc. If this goes closed source
// it will have to be re-implemented.
import java.util.Arrays;
import java.io.Serializable;

/** Used to ID a resource.  Currently the SHA hash of the file. This version
 *  is some what simplified from the OpenCola version of ResourceId.  
 *
 * @author Ry4an
 * @author Luke Francl
 */
public class ResourceId implements Serializable {

    /** Length of a resource id in bytes. */
    public static final int LENGTH = 20;

    private byte[] id;
   
    /** @param b the SHA hash of the file. **/
    public ResourceId(byte[] b) {
	this.id = b;
    }

    public ResourceId(String hexString) {
	this.id = Util.hexToBytes(hexString);
    }
    
    public boolean equals(Object o) {
	if ((o == null) || ! (o instanceof ResourceId)) {
	    return false;
	}
	return Arrays.equals(((ResourceId)o).id, this.id);
    }

    public int hashCode() { 
	return (this.toString()).hashCode();
    }

    public String toString() {
        return Util.bytesToHex(id);
    }
}
