/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.util;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Collection;
import java.util.Set;

import org.francl.eikon.image.Image;

/** 
    The search array is a construct which holds matches for a certain 
    coeffient value to speed searching.
    
    @author Luke Francl
**/
public class SearchArray implements Serializable {
    public static final boolean DEBUG = false;

    /** Holds the 2D array of HashSet objects. Each x,y coordinate will
	contain a HashSet of all the ResourceIds with a truncated,
	quantized wavelet coeffient at that location. **/
    private HashSet[][] search;


    /** Do pre-initialization of the bins in this SearchArray **/
    public SearchArray() {
	System.out.println("SearchArray: creating new SearchArray");
	search =
	    new HashSet[Image.SCALED_IMAGE_WIDTH][Image.SCALED_IMAGE_HEIGHT];
	for (int y = 0; y < search.length; y++) {
	    for (int x = 0; x < search[0].length; x++) {
		search[y][x] = new HashSet();
	    }
	}
    }

    /** Adds a new ResouceId to the ArrayList at position x, y in the 
	search array.
    **/
    public void add(ResourceId id, int x, int y) {
	if(DEBUG) {
	    String m = "In SearchArray.add. ResourceId = "+id+" x="+x+" y="+y;
	    System.out.println(m);
	}
	(search[y][x]).add(id);
	if(DEBUG) {
	    String m = "SearchArray.add: search["+y+"]["+x+"] = " +
		search[y][x];
	    System.out.println(m);
	}

    }

    /** Removes a ResouceId from any positions it may occupy in the
	SearchArray.
    **/
    public void remove(ResourceId id) {
	for(int y = 0; y < search.length; y++) {
	    for (int x =0; x < (search[0]).length; x++) {
		search[y][x].remove(id);
	    }
	}
    }

    /** Removes a ResourceId from the HashSet at position x, y in the
	search array.
    **/
    public void remove(ResourceId id, int x, int y) {
        Set target = search[y][x];
	target.remove(id);
    }

    /** Removes all the entries at (x, y) **/
    public void clear(int x, int y) {
	(search[y][x]).clear();
    }

    /** Returns a Collection filled with of all the ResouceIds with a match at 
	position (x, y).
    **/
    public Collection matches(int x, int y) {
	Set matches = search[y][x];
	if (DEBUG) {
	    String m = "SearchArray.matches: matches for (x="+x+",y="+y+"): " +
		matches;
	    System.out.println(m);
	}

	return matches;
    }
}
