/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.util;

// Some of these methods are from GPL code (c) 2001 OpenCola, Inc. They'll
// have to be re-written if this goes closed source.

/** The Util class contains helper routines that have re-use potential, or
    just make my code easier to read, componetize, and debug. 
    @author Luke Francl
    @author Ry4an Brase
**/
public class Util {

    private static char[] hexDigit = new char[] 
    {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

    /** Methods to print double and int 1 and 2D arrays **/
    public static void printArray(double[] arr) {
	for(int i=0;i<arr.length;i++) {
	    System.out.print(" " + arr[i] + " ");
	}
	System.out.print("\n");
    }

    public static void printArray(int[] arr) {
	for(int i=0;i<arr.length;i++) {
	    System.out.print(" " + arr[i] + " ");
	}
	System.out.print("\n");
    }

    public static void printArray(double[][] arr) {
	for(int i=0; i<arr.length; i++) {
	    printArray(arr[i]);
	}
    }

    public static void printArray(int[][] arr) {
	for(int i=0; i<arr.length; i++) {
	    printArray(arr[i]);
	}
    }
    /** Turn a byte array into a lowercase hex string. Wrapper for
	bytesToHex that converts the entire array.
     */
    public static String bytesToHex(byte[] in) {
        return bytesToHex(in,0,in.length);
    }

    /** Turn a byte array into a lowercase hex string
     */
    public static String bytesToHex(byte[] in, int off, int len) {
        char[] out = new char[in.length * 2];
	for (int i = 0; i < len; i++) {
            out[i*2] = hexDigit[(0xF0 & in[i+off]) >> 4]; // high nybble
	    out[i*2+1] = hexDigit[0xF  & in[i+off]]; // low nybble
	}
	return new String(out);
    }

    /** Create a byte array from a hex string
     */
    public static byte[] hexToBytes(String in) {
        int len = in.length();
    	if (len % 2 != 0) {
	    throw new IllegalArgumentException("Even length string expected.");
	}
	byte[] out = new byte[len/2];
	try {
	    for (int i = 0; i < out.length; i++) {
		out[i] =
		    (byte)(Integer.parseInt(in.substring(i*2, i*2+2), 16));
	    }
	} catch (NumberFormatException doh) {
	    doh.printStackTrace();
	    throw new IllegalArgumentException("ParseError");
	}
	return out;
    }

    /** A lot of methods need to convert a one character long string to
	a char, or throw an error if there is one. This utility method
	does just this.
	@returns a the first character of the String as a char
	@throws IllegalArgumentException if the string has more than one
	character in it.
    **/
    public static char stringToChar(String s) {
	if (s.length() > 1) {
	    String msg = s + " has more than one character. This is illegal.";
	    throw new IllegalArgumentException(s);
	}

	return s.charAt(0);
    }

    /** Escapes "&" character with it's XML appropriate replacement "&amp;"
	This is really buggy because if you give it a string that ALREADY
	contains "&amp;" it'll screw up. Quick-and-dirty...
     **/
    public static String escapeAmp(String s) {
	if(s.indexOf('&') != -1) {
	    String[] results = s.split("&");
	    StringBuffer sb = new StringBuffer();
	    for(int i=0; i < results.length; i++) {
		sb.append(results[i]);
		if(i < results.length - 1) {
		    sb.append("&amp;");
		}
	    }
	    return sb.toString();
	} else {
	    return s;
	}
    }
}
