/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.wavelet;

import org.francl.eikon.image.Image;

public class Quantizor {
    
    public Signature quantize(Image image) {
	String colorspace = image.getColorSpace();
	char firstColor = colorspace.charAt(0);
	char secondColor = colorspace.charAt(1);
	char thirdColor = colorspace.charAt(2);
	
	double[][] firstTrunc  = image.getElements(firstColor);
	double[][] secondTrunc = image.getElements(secondColor);
	double[][] thirdTrunc  = image.getElements(thirdColor);
	
	// initalize arrays to all zeros
	int[][] firstQuant = new int
	    [firstTrunc.length]			
	    [(firstTrunc[0]).length];
	
	int[][] secondQuant = new int
	    [secondTrunc.length]
	    [(secondTrunc[0]).length];
	
	int[][] thirdQuant = new int
	    [thirdTrunc.length]
	    [(thirdTrunc[0]).length];

	// quantize the coeffients
	for(int i=0; i < firstTrunc.length; i++) {
	    for(int j=0; j < (firstTrunc[0]).length; j++) {
		firstQuant[i][j]  = doQuantize(firstTrunc[i][j]);
		secondQuant[i][j] = doQuantize(secondTrunc[i][j]);
		thirdQuant[i][j]  = doQuantize(thirdTrunc[i][j]);
	    }
	}

	// FIXME: This needs to actually work...
	Signature sig = new YIQSignature(firstQuant, secondQuant, thirdQuant,
					 image.getAverage('Y'),
					 image.getAverage('I'),
					 image.getAverage('Q'));
	
	return sig;
    }
		
    /** Does quantization on a singleton double 
	@param val the value to quantize
	@returns 1 if val is greater than 0.0, -1 if val is less than 0.0,
	or 0 if val equals 0.0
    **/
    private int doQuantize(double val) {
	if(val > 0d) {
	    return 1;
	} else if(val < 0d) {
	    return -1;
	} else {
	    return 0;
	}
    }
}
