/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.wavelet;

import java.util.Iterator;

/** The Signature class contains the data and methods to represent and
    deal with Eikon's wavelet decomposition image signatures. It is basically
    the same as the old TruncatedDecomposedImage class, with a better
    name. It also is more abstract, with fields for colorspace, signature size
    and wavelet decomposition type.

    <p>Signature objects can be created from their XML representations, 
    turned into an XML representation, inserted into the database, and used 
    by Query objects to get results from the database.
    
    @author Luke Francl
    
**/

public interface Signature {

    public int[][] getCoeffients(String colorChannel)
	throws ColorChannelNotSupportedException;
    
    public int[][] getCoeffients(char colorChannel) 
	throws ColorChannelNotSupportedException;

    public double getAverage(String colorChannel)
	throws ColorChannelNotSupportedException;

    public double getAverage(char colorChannel) 
	throws ColorChannelNotSupportedException;
    
    public Iterator getColors();
    
    public String getColorSpace();
    
    public String getWaveletBasis();

    public double[] getWeights(char color)
	throws ColorChannelNotSupportedException;

    public double[] getWeights(String color)
	throws ColorChannelNotSupportedException;
    
    public int getWidth();
    
    public int getHeight();
    
    public String toXml();
    
    public String toString();
}
