/*
  Eikon is an image similarity search engine based on wavelets.
  Copyright (C) 2002 Luke Francl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package org.francl.eikon.wavelet;

import org.francl.eikon.image.Image;

public class Truncator {
    public Image truncate(Image image, int largest) {
	String colorspace = image.getColorSpace();
	char firstColor  = colorspace.charAt(0);
	char secondColor = colorspace.charAt(1);
	char thirdColor  = colorspace.charAt(2);

		
	double[][] firstElem  = image.getElements(firstColor);
	double[][] secondElem = image.getElements(secondColor);
	double[][] thirdElem  = image.getElements(thirdColor);

	// set average color position (which is located at (0,0) to 0 so 
	// it doesn't mess up the search for the largest wavelet coeffients.
	
	firstElem[0][0]  = 0d;
	secondElem[0][0] = 0d;
	thirdElem[0][0]  = 0d;
	
	CoeffientList firstCoeffientList  = new CoeffientList(firstElem);
	CoeffientList secondCoeffientList = new CoeffientList(secondElem);
	CoeffientList thirdCoeffientList  = new CoeffientList(thirdElem);
	
	firstElem  = firstCoeffientList.getLargest(largest);
	secondElem = secondCoeffientList.getLargest(largest);
	thirdElem  = thirdCoeffientList.getLargest(largest);
	
	image.setElements(firstColor, firstElem);
	image.setElements(secondColor, secondElem);
	image.setElements(thirdColor, thirdElem);
	
	return image;
    }
}
